// filename ******** Main.C ************** 
// Example main program for OutputCompare interrupt example 
// Technological Arts MC9S12C32 board  
// TCNT increments at a fixed rate, OC is a periodic interrupt
// PT1 and PT0 are outputs to LEDs
// PAD6 and PAD7 are inputs from switches
// if both buttons pushed, alternate LED toggle
// if PAD7 button pushed, both LEDs are on
// if PAD6 button pushed, both LEDs are off
// if both buttons not pushed, together LED toggle

//  This example accompanies the books
//   "Embedded Microcomputer Systems: Real Time Interfacing", 
//     Thomsen, copyright (c) 2006,
//   "Introduction to Embedded Microcomputer Systems: 
//    Motorola 6811 and 6812 Simulation", Brooks-Cole, copyright (c) 2002,
//    Jonathan W. Valvano 3/17/07

// You may use, edit, run or distribute this file 
//    as long as the above copyright notices remain
#include <hidef.h>      /* common defines and macros */
#include <mc9s12c32.h>     /* derivative information */


#pragma LINK_INFO DERIVATIVE "mc9s12c32"
#include "pll.h"

const struct State{
  unsigned char Out;            // Output to Port T
  const struct State *Next[4];  // Next state if input=0,1,2,3
};
typedef const struct State StateType;
typedef StateType *	StatePtr;

#define SA &fsm[0]
#define SB &fsm[1]
#define SC &fsm[2]
#define SD &fsm[3]
#define SE &fsm[4]
#define SF &fsm[5]
StateType fsm[6]={
	{0x01,{SB,SC,SD,SE}},  // SA,SB alternate toggle
	{0x02,{SA,SC,SD,SE}},  // SB
	{0x03,{SA,SC,SD,SE}},  // SC both on
	{0x00,{SA,SC,SD,SE}},  // SD both off
	{0x00,{SA,SC,SD,SF}},  // SE,SF together toggle
	{0x03,{SA,SC,SD,SE}}   // SF
};
StatePtr Pt;  // Current State 
unsigned char Input;

interrupt 11 void TOC3handler(void){
  PTT = Pt->Out;         // Output depends on the current state
  Input = PTAD>>6;       // Input=0,1,2,or 3
  Pt = Pt->Next[Input];  // Next state depends on the input 
  TC3 = TC3+46875U;      // every 0.25s at 24 MHz 
  TFLG1 = 0x08;          // acknowledge OC3
}

//--------------OC3_Init----------------
// enable 0.25sec periodic interrupt
// Inputs: none
// Outputs: none
void OC3_Init(void){
  TIOS |= 0x08;   // activate TC3 as output compare
  TSCR1 = 0x80;   // Enable TCNT, 24MHz
  TSCR2 = 0x07;   // divide by 128 TCNT prescale, TOI disarm
  PACTL = 0;      // timer prescale used for TCNT
/* Bottom three bits of TSCR2 (PR2,PR1,PR0) determine TCNT period
    divide  FastMode(24MHz)    Slow rate (4MHz)
000   1     42ns  TOF  2.73ms  250ns TOF 16.384ms
001   2     84ns  TOF  5.46ms  500ns TOF 32.768ms 
010   4    167ns  TOF  10.9ms    1us TOF 65.536ms   
011   8    333ns  TOF  21.8ms 	 2us TOF 131.072ms
100  16    667ns  TOF  43.7ms 	 4us TOF 262.144ns
101  32   1.33us  TOF  87.4ms		 8us TOF 524.288ms
110  64   2.67us  TOF 174.8ms   16us TOF 1.048576s
111 128   5.33us  TOF 349.5ms   32us TOF 2.097152s */ 
  TIE  |= 0x08;    // arm OC3
  TC3   = TCNT+50; // first interrupt right away
asm cli            // enable interrupts 
}
void main(void) {
  PLL_Init();      // running at 24MHz
  OC3_Init();
  DDRT |= 0x03;
  DDRAD = 0x00;   // PortAD are inputs, PortT is output
  ATDDIEN = 0xFF; // enable AD digital input drivers  
  Pt = SA;        // Initial State
  for(;;){
  }
}